/*
    tcptest - Web server stress test
    Copyright (C) 1999  Scott Donoho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "tcptest.h"

extern int errno;
extern int vflag;

void sig_client(int);
void sig_client_start(int);
void http_conn(char*,int,int,int,int,int,int);
void copyleft();
long parse_num(char *);
int statp[2];
int mypid,done;

client_start(char *host, int port, int num_stripe, int bsize, int num_conn, int num_secs, int WMax, int WMin)
{
	struct status CRet;
	struct timeval bstart, bstop;
	double cps,seconds,secs,micros,startu,stopu;
	pid_t *pids;
	int i,pid,hours,mins;
	int reporting = 0;
	int total_con = 0;

	mypid = getpid();

	/* Signaling */
	if(signal(SIGINT, sig_client) == SIG_ERR){
		perror("signal");
		exit(-1);
	}

	if(signal(SIGALRM, sig_client) == SIG_ERR){
		perror("signal");
		exit(-1);
	}

	if((pipe(statp))){
		perror("pipe");
		exit(-1);
	}

	copyleft();
	pids = (pid_t *)malloc(sizeof(int)*num_stripe);
	printf("Creating clients...\n");
	for(i=0;i<num_stripe;i++){
		if((pid = fork()) < 0){ /* Error */
			perror("fork");
			exit(-1);
		}
		else if(pid == 0){ /* Child */
			mypid = getpid();
			http_conn(host,port,bsize,num_secs,num_conn,WMax,WMin);
			exit(0);
		}
		/* Parent */
		*(pids+i) = pid;
		if(vflag)
			printf("Starting process: %d\n", pid);
	}

	/* Jump start all of the child processes */
	sleep(2);
	printf("Starting clients...\n");
	for(i=0;i<num_stripe;i++){
		kill(*(pids+i),SIGHUP);
	}

	bstop.tv_sec = 0;
	bstop.tv_usec = 0;
	bstart.tv_sec = 0;
	bstart.tv_usec = 0;
	printf("Waiting on clients...\n");
	for(i=0;i<num_stripe;i++){
		readn(statp[0],&CRet,sizeof(CRet));
		reporting++;
		secs = CRet.stop.tv_sec - CRet.start.tv_sec;
		micros = CRet.stop.tv_usec - CRet.start.tv_usec;
		micros = micros + (secs * MICRO);
		/* Save the earliest start and latest stop time */
		if(CRet.start.tv_sec == bstart.tv_sec)
			if(CRet.start.tv_usec < bstart.tv_usec)
				bstart.tv_usec = CRet.start.tv_usec;
		if(CRet.start.tv_sec < bstart.tv_sec || bstart.tv_sec == 0){
			bstart.tv_sec = CRet.start.tv_sec;
			bstart.tv_usec = CRet.start.tv_usec;	
		}
		if(CRet.stop.tv_sec > bstop.tv_sec){
			bstop.tv_sec = CRet.stop.tv_sec;
			bstop.tv_usec = CRet.stop.tv_usec;	
		}
		seconds = micros / MICRO;
		cps = CRet.connects/seconds; 
		total_con = total_con + CRet.connects;
		printf("%4d Client(%d): %d conn, %4.2f secs, %4.2f cps\n",
					reporting,CRet.pid,
					CRet.connects,seconds,cps);
	}
	startu = (bstart.tv_sec * MICRO) + bstart.tv_usec;
	stopu = (bstop.tv_sec * MICRO) + bstop.tv_usec;
	secs = stopu - startu;
	seconds = micros/1000000.0;
	cps = total_con/seconds;
	printf("Totals:\n");
	printf("\tConnections: %d\n",total_con);
	hours=seconds/3600;
	mins=(seconds-(hours * 3600))/60;
	secs = seconds - (hours*3600) - (mins * 60);
	printf("\tTime: %02d:%02d:%02.2f(%.2f seconds)\n",hours,mins,secs,seconds);
	printf("\tConnect/Sec: %4.2f\n",cps);
}

void http_conn(char *host,int port,int bsize,int ntime,int nconn,int max, int min)
{
	struct sockaddr_in name;
	struct hostent *hp, *gethostbyname();
	struct status CRet;
	struct timeval tx;
	struct timezone tzone;
	char wbuf[] = {'G','E','T',' ','/','\n','\0'};
	char rbuf[MAXBUFSZ];
	int sock,i,bytes,seed,sleep_time;
	int delay =0;
	int connections = 0;

	/*  
	 * max wait is set (wait between max and min secs for connections)
	 */ 
	if(max > 0)
		delay = 1;

	if(signal(SIGUSR1, sig_client_start) == SIG_ERR){
		perror("signal");
		exit(-1);
	}

	/* Wait to start */
	pause();

	if(ntime){
		alarm(ntime);
		done = 0;
	}
	else if(nconn){
		done = 0;
	}
	else{
		done = 1;
	}
		
	gettimeofday(&CRet.start,&tzone);
	while(!done){
		if(delay){
			gettimeofday(&tx,&tzone);
			seed = tx.tv_usec & 0xffff;
			sleep_time = rand_int(seed,min,max);
			if(sleep_time)
				sleep(sleep_time);
		}
		/* Create a socket */
		sock = socket(AF_INET, SOCK_STREAM, 0);
		if(sock < 0){
			perror("socket");
			exit(-1);
		}

		connections++;

		/* Resolve host */
       		hp = gethostbyname(host);
		if(hp == 0){
			printf("%s: unknown host\n",host);
			close(sock);
			exit(-2);
		}
	
		bcopy(hp->h_addr, &name.sin_addr, hp->h_length);
		name.sin_family = AF_INET;
		name.sin_port = htons(port);
	
		/* Make http connection */
		if(connect(sock,(struct sockaddr *)&name, sizeof(name)) < 0){
			perror("connect");
			close(sock);
			exit(-3);
		}


		/*
		 * Send Http Request
		 */
		bytes = writen(sock,&wbuf,sizeof(wbuf));
		
		/* read the request */
		bytes = readn(sock,&rbuf,bsize);

        	close(sock);
		if(nconn)
			if(nconn == connections)
				done = 1;
	}
	/* Report results back to the main client */
	gettimeofday(&CRet.stop,&tzone);
	CRet.pid = mypid;
	CRet.connects = connections;
	writen(statp[1],&CRet,sizeof(CRet));
}

void sig_client(int signo)
{
	done = 1;
}

void sig_client_start(int signo)
{
	if(vflag)
		printf("starting %d\n",mypid);
}

void copyleft()
{
	printf("tcptest version 1.0, Copyright(C) 1999 Scott Donoho\n");
	printf("tcptest comes with ABSOLUTELY NO WARRANTY \n");
	printf("This is free software, and you are welcome to redistribute it\n");
	printf("under certain conditions; see COPYING file included in this\n");
	printf("distribution\n");
}
