/*
    tcptest - Web server stress test
    Copyright (C) 1999  Scott Donoho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/                                        

/*
 * Http network test client
 *
 *	Author: Scott Donoho
 *	Date: 8/99
 *
 *	tcptest [-Sv] 
 *              [-p port#] [-d blocksiz] 
 *              [-s stripe] [-n connects]
 *		[-t seconds] 
 *              serverhostname
 *
 *	defaults:
 *		-p 80
 *		-d 32k (no data xfer back after connection
 *		-c 0
 *		-s 1
 *		-n 0
 *		-t 0
 *		-v off
 *
 */

#include "tcptest.h"

extern char *optarg;
extern int optind;
extern int errno;

void usage();
void sig_main(int);
char opt[] = {"SW:b:d:n:p:s:t:vw:"};
int done,vflag;

main(argc,argv)
        int argc;
        char *argv[];
{
	char host[128];
	int getopt(), optchr;
	int bsize,num_conn_max,num_stripe,num_secs,port;
	int i,pid,Sflag;
	int WMax,WMin;

	/* Initialize variables */
	port = PORT;
	done = 0;
	vflag = DEBUG;
	Sflag = 0;
	WMax = 0;
	WMin = 0;
	bsize = MAXBUFSZ;
	num_conn_max = num_secs = 0;
	num_stripe = 1;

	while((optchr = getopt(argc,argv,opt)) != EOF){
		switch(optchr){
			case 'S':
				Sflag++;
				break;

			case 'W':
				WMax = atoi(optarg);
				break;

			case 'd':
				bsize = parse_num(optarg);
				if(bsize > MAXBUFSZ || bsize == -1){
					usage(argv[0]);
					exit(-1);
				}
				break;

			case 'n':
				num_conn_max = parse_num(optarg);
				break;

			case 'p':
				port = atoi(optarg);
				break;

			case 's':
				num_stripe = atoi(optarg);
				break;

			case 't':
				num_secs = atoi(optarg);
				break;

			case 'v':
				vflag++;
				break;

			case 'w':
				WMin = atoi(optarg);
				break;

			case '?':
				usage(argv[0]);
				exit(-1);
		}
	}

	if(WMax < WMin){
		fprintf(stderr,"Max wait must be > Min wait\n");
		usage(argv[0]);
		exit(-1);
	}

	if(optind < argc)
		strcpy(host,argv[optind]);
	else if(!Sflag){
		usage(argv[0]);
		exit(-1);
	}

	if(vflag){
		if(!Sflag)
			printf("host: %s\n",host);
		printf("port: %d\n",port);
		printf("data size: %d\n",bsize);
		printf("Num Connects: %d\n",num_conn_max);
		printf("Total Seconds: %d\n",num_secs);
		printf("Stripe Count: %d\n\n",num_stripe);
	}

	if(signal(SIGHUP, sig_main) == SIG_ERR){
		perror("signal");
		exit(-1);
	}

	if(Sflag)
		server_start(port,bsize);
	else
		client_start(host,port,num_stripe,bsize,num_conn_max,num_secs,WMax,WMin);
}

/* Toggle verbose and reset stats */
void sig_main(int signo)
{
	vflag = vflag?0:1;
}

void usage(char *cmd)
{
	fprintf(stderr,"%s [-Sv] \n",cmd);
	fprintf(stderr,"\t[-d blksz] \n");
	fprintf(stderr,"\t[-n number of connections] \n");
	fprintf(stderr,"\t[-p port number] \n");
	fprintf(stderr,"\t[-s number of clients]\n");
	fprintf(stderr,"\t[-t secs]\n");
	fprintf(stderr,"\t<server name>\n");
}
