/*
    tcptest - Web server stress test
    Copyright (C) 1999  Scott Donoho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/                                        

#include "tcptest.h"

extern int errno;
extern int vflag;

void sig_server(int);
long parse_num(char *);

char rbuf[MAXBUFSZ];
char wbuf[MAXBUFSZ];
int sock,newfd;

/*
 * TCPTEST server mode
 *	listen port number
 * 	data size of reply
 */
server_start(int port, int bsize)
{
        struct sockaddr_in serv_addr, cli_addr;
	struct linger ling;
	int connects = 0;
	int i,pid,fd;
	int lsr_size, lss_size;
	int sock_opt_len = 0;

	/* Don't wait for children */	
	if(signal(SIGCHLD, SIG_IGN) == SIG_ERR){
		perror("signal");
		exit(-1);
	}

	if(signal(SIGINT, sig_server) == SIG_ERR){
		perror("signal");
		exit(-1);
	}

        if((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0){
                perror("opening socket");
                exit(1);
        }

	bzero((char*) &serv_addr, sizeof(serv_addr));
        serv_addr.sin_family = AF_INET;
        serv_addr.sin_addr.s_addr = INADDR_ANY;
        serv_addr.sin_port = htons(port);
        if(bind(sock, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0){
                perror("binding socket");
                close(sock);
		exit(-1);
        }

	/* Get and Set socket options */
	i = sizeof(ling);
	if(getsockopt(sock,SOL_SOCKET,SO_LINGER,&ling,&i) != 0 ){
		perror("getsockopt");
		close(sock);
		exit(-1);
	}

	if(getsockopt(sock,SOL_SOCKET,SO_SNDBUF,
			(char *)&lss_size, &sock_opt_len) < 0){
		perror("getsockopt");
		close(sock);
		exit(-1);
	}

	if(getsockopt(sock,SOL_SOCKET,SO_RCVBUF,
			(char *)&lsr_size, &sock_opt_len) < 0){
		perror("getsockopt");
		close(sock);
		exit(-1);
	}

	if(vflag){
		printf("linger: %d/%d\n",ling.l_onoff,ling.l_linger);
		printf("sndbuf: %d, rcvbuf: %d\n", lss_size, lsr_size);
	}

	/* Start listening for and accepting client connections */
	if(listen(sock,SOMAXCONN)){
		perror("listen:");
	}

	for(;;){
		i = sizeof(cli_addr);
		if((newfd = accept(sock,(struct sockaddr *)&cli_addr,&i))<0){
			perror("accept");
			close(sock);
                	exit(-1);
		}
		else{ 

			/* printf("Connect!\n"); */
			connects++;
		}

		if(newfd < 0){
			printf("Accept error\n");
			close(sock);
		}

		if((pid = fork()) < 0){ /* Error */
			perror("fork");
			close(sock);
			exit(-2);
		}
		else if(pid == 0){ /* Child */
			close(sock);
			proc_client(newfd,bsize);
			close(newfd);
			_exit(0);
		}
		close(newfd); /* Parent */

		if(vflag)
			if((connects % 10) == 0 )
				printf("con = %d\n",connects);
	}
}

int proc_client(int s,int bsize)
{
	int bytes;

	/* Read request from client */
	if((bytes = read(s,&rbuf,sizeof(rbuf))) < 0){
		perror("read");
		close(newfd);
		exit(-3);
	}
	/* Write blocks back to client */
	bytes = writen(s,&wbuf,bsize);
}

void sig_server(int signo)
{
	printf("Bye!\n");
	close(sock);
	_exit(1);
}
